package com.qq.taf.net;

/**
 * 序列同步器
 * @author fanzhang
 */
public class ObjectSequenceSynchronizer<T> {
	
	private static final long MAX_TIMEOUT=Long.MAX_VALUE;
	private static final int DEFAULT_CAPACITY=1024;
	private long timeout;
	private int capacity;
	private ObjectSynchronizer<T>[] array;
	
	public ObjectSequenceSynchronizer() {
		this(MAX_TIMEOUT,DEFAULT_CAPACITY);
	}
	
	public ObjectSequenceSynchronizer(long timeout) {
		this(timeout,DEFAULT_CAPACITY);
	}
	
	@SuppressWarnings("unchecked")
	public ObjectSequenceSynchronizer(long timeout,int capacity) {
		this.timeout=timeout;
		int n=2;
		while(n<capacity) n=n<<1;
		this.capacity=n;
		array=new ObjectSynchronizer[this.capacity];
		for(int i=0;i<this.capacity;i++) array[i]=new ObjectSynchronizer<T>();
	}
	
	public void init(int seqid) {
		array[seqid & (capacity-1)].init();
	}

	public void put(int seqid,T obj) {
		array[seqid & (capacity-1)].put(obj);
	}
	
	public T get(int seqid) {
		return array[seqid & (capacity-1)].get(timeout);
	}
	
}
