package com.qq.taf.net;

import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.TimeUnit;

/**
 * 连接获取同步器
 * @author fanzhang
 */
public class SessionSynchronizer {

	private static final long MAX_TIMEOUT=Long.MAX_VALUE;
	private static final int DEFAULT_CAPACITY=1024;
	private long timeout;
	private int capacity;
	private SynchronousQueue<Session>[] array;
	
	public SessionSynchronizer() {
		this(MAX_TIMEOUT,DEFAULT_CAPACITY);
	}
	
	public SessionSynchronizer(long timeout) {
		this(timeout,DEFAULT_CAPACITY);
	}
	
	@SuppressWarnings("unchecked")
	public SessionSynchronizer(long timeout,int capacity) {
		this.timeout=timeout;
		int n=2;
		while(n<capacity) n=n<<1;
		this.capacity=n;
		array=new SynchronousQueue[this.capacity];
		for(int i=0;i<this.capacity;i++) array[i]=new SynchronousQueue<Session>();
	}
	
	public Session get(int sessionId) {
		Session obj=null;
		try {
			obj=array[sessionId & (capacity-1)].poll(timeout, TimeUnit.MILLISECONDS);
		} catch(Throwable t) {
		} 
		return obj;
	}
	
	public void put(Session session) {
		try {
			boolean ok=array[session.getID() & (capacity-1)].offer(session, timeout, TimeUnit.MILLISECONDS);
			if(!ok) session.close(true);
		} catch(Throwable t) {
		}
	}
	
}
