package com.qq.taf.net.file;

import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.WritableByteChannel;

public class NetFileBuffer implements NetDataBuffer{
	
	private long offset;
	
	private long length;
	
	private RandomAccessFile file;
	
	private FileChannel channel;
	
	public NetFileBuffer(String fileName, long offset, long length) throws Throwable {
		this.offset = offset;
		this.length = length;
		file = new RandomAccessFile(fileName, "r");
		channel = file.getChannel();
	}
	
	@Override
	public boolean hasRemaining() {
		return length > 0;
	}

	@Override
	public long write(WritableByteChannel target) throws Throwable {
		long len = 0;
		try {
			len = channel.transferTo(offset, length, target);
			offset += len;
			length -= len;
		}
		catch(Throwable th) {
			th.printStackTrace();
			throw th;
		}
		return len;
	}

	@Override
	public long remaining() {
		return length;
	}

	@Override
	public void close() throws Throwable {
		if(file != null) {
			file.close();
		}
	}
	
}
