package com.qq.taf.net.file;

import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;
import java.util.concurrent.ConcurrentLinkedQueue;

import com.qq.taf.net.MessageReceiver;
import com.qq.taf.net.HandlerExecutor;
import com.qq.taf.net.Logger;
import com.qq.taf.net.Processor;
import com.qq.taf.net.Ternary;

/**
 * 网络IO处理器
 * @author fanzhang
 */
public class NetProcessor extends Processor implements Runnable{
	private NetFilter filter;
	
	public NetProcessor(String name,NetFilter filter,NetFilterEncodeProduct filterEncodeProduct,MessageReceiver filterDecodeProduct,HandlerExecutor handlerExecutor,int num) {
		super(name,filter,filterEncodeProduct,filterDecodeProduct,handlerExecutor,num);
		this.filter = filter;
	}
	
	protected void write(SelectionKey key,long now) {	
		if(!key.isValid()) return;
		NetSession session=(NetSession)key.attachment();
		try {
			session.scheduleWrite(false);
			setInterestedInWrite(key, false);
			SocketChannel channel=(SocketChannel)key.channel();
			int write=0;
			ConcurrentLinkedQueue<NetDataBuffer> queue=session.getNetWriteQueue();
			NetDataBuffer buff=null;
			while((buff=queue.peek())!=null) {
				if(buff==NetDataBuffer.CLOSE_DATA) {
					queue.clear();
					scheduleCancel(key);
					buff=null;
					break;
				}
				long n=0;
				for(int i=0;i<WRITE_SPIN_COUNT;i++) if((n=buff.write(channel))>0) break;
				if(n==0) break;
				write+=n;
				if(!buff.hasRemaining()) {
					queue.poll();
					buff.close();//销毁文件资源
				}
			}
			if(write>0) {
				session.setLastWriteTime(now);
				session.increaseWriteBytes(write);
			} 
			if(buff!=null) {
				setInterestedInWrite(key, true);
				session.scheduleWrite(true);
			}
		} catch(Throwable t) {
			session.close(true);
			handlerExecutor.exceptionCaught(session, t);
		}
	}
	
	protected void register(Ternary<Integer, SocketChannel,Integer> registerTernary,long now) {
		try {
			int sessionId=registerTernary.first;
			SocketChannel socketChannel=registerTernary.second;
			int ioTimeoutMillis=registerTernary.third;
			socketChannel.configureBlocking(false);
			socketChannel.socket().setReuseAddress(true);
			socketChannel.socket().setTcpNoDelay(true);
			socketChannel.socket().setKeepAlive(true);
			SelectionKey key=socketChannel.register(selector, SelectionKey.OP_READ);
			NetSession session=new NetSession(sessionId,key,this,filter,(NetFilterEncodeProduct)filterEncodeProduct,handlerExecutor,ioTimeoutMillis);
			key.attach(session);
			session.setOpenTime(now);
			handlerExecutor.sessionOpened(session);
		} catch (Throwable t) {
			Logger.log("processor register error",t);
		}
	}
	
}
