package com.qq.taf.net.file.test;

import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.util.concurrent.atomic.AtomicInteger;

import com.qq.jutil.j4log.Logger;
import com.qq.taf.net.MessageReceiver;
import com.qq.taf.net.Handler;
import com.qq.taf.net.Session;
import com.qq.taf.net.SessionSynchronizer;
import com.qq.taf.net.file.NetByteBuffer;
import com.qq.taf.net.file.NetConnector;
import com.qq.taf.net.file.NetFileBuffer;
import com.qq.taf.net.file.NetFilter;
import com.qq.taf.net.file.NetFilterEncodeProduct;
import com.qq.taf.net.file.NetMixBuffer;
import com.qq.taf.net.file.NetSession;

public class EchoClient {
	static int count = 0;
	private static interface Counter {
		public int getAndIncrement();
	}
	
	private static class MultiThreadsCounter implements Counter {
		
		private AtomicInteger counter=new AtomicInteger(0);

		public int getAndIncrement() {
			return counter.getAndIncrement();
		}
		
	}
	
	private static class SingleThreadCounter implements Counter {
		
		private int counter=0;

		public int getAndIncrement() {
			return counter++;
		}
		
	}
	
	public static void main(String[] args) {
		String ip=System.getProperty("ip","127.0.0.1");
		int port=Integer.parseInt(System.getProperty("port","12345"));
		final InetSocketAddress remote=new InetSocketAddress(ip,port);
		int threads=Integer.parseInt(System.getProperty("threads","1"));
		int handleThreads=1;
		final int loop=Integer.parseInt(System.getProperty("loop","100"));
		final int batch=Integer.parseInt(System.getProperty("batch","200"));
		final int sleep=Integer.parseInt(System.getProperty("sleep","2"));
		
		int size=32;
		final byte[] DATA=new byte[size];
		for(int i=0;i<size;i++) DATA[i]='I';
		
		final Logger LOGGER=Logger.getLogger("EchoClient");
		
		final SessionSynchronizer sessionSynchronizer=new SessionSynchronizer(1000,1);
//		final ObjectSequenceSynchronizer<EchoResponse> invocationSynchronizer=new ObjectSequenceSynchronizer<EchoResponse>(1000,32768);
		
		final NetConnector connector=new NetConnector(
			"EchoClient",
			new NetFilter() {
				public void encode(NetSession session, Object object,NetFilterEncodeProduct product) throws Throwable {
					EchoRequest res=(EchoRequest)object;
					NetFileBuffer buffer = new NetFileBuffer("e:/data", 1L, (int)(8-1) );
					ByteBuffer bb = ByteBuffer.allocate(4+4);
					bb.putInt(8-1+8).putInt(res.seqid).flip();
					NetByteBuffer nbb = new NetByteBuffer(bb);
					product.add(session, new NetMixBuffer(nbb, buffer));
				}
				public void decode(Session session, ByteBuffer buffer,MessageReceiver product) throws Throwable {
					ByteBuffer now=buffer;
					ByteBuffer prev=(ByteBuffer)session.getAttribute("buff");
					if(prev!=null) {
						session.removeAttribute("buff");
						now=(ByteBuffer)ByteBuffer.allocate(prev.remaining()+buffer.remaining()).put(prev).put(buffer).flip();
					}
					while(now.remaining()>4 && now.remaining()>=now.getInt(now.position())) {
						int len=now.getInt();
						int seqid=now.getInt();
						byte[] data=new byte[len-4-4];
						now.get(data);
						EchoResponse res=new EchoResponse();
						res.seqid=seqid;
						res.data=data;
						product.add(session, res);
					} 
					if(now.hasRemaining()) {
						ByteBuffer succ=(ByteBuffer)ByteBuffer.allocate(now.remaining()).put(now).flip();
						session.setAttribute("buff", succ);
					}
				}
			},
			new Handler() {
				@Override
				public void sessionOpened(Session session) throws Throwable {
					sessionSynchronizer.put(session);
				}
				public void messageRecieved(Session session, Object message) throws Throwable {
					EchoResponse res=(EchoResponse)message;
//					invocationSynchronizer.put(res.seqid, res);
					LOGGER.info("res|"+res.seqid+"|"+new String(res.data));
					System.out.println(++count);
				}
				public void exceptionCaught(Session session, Throwable exception) throws Throwable {
					session.close(true);
				}
			},
			0,
			handleThreads
		);
		
		System.out.println("EchoClient ...");
		final Counter COUNTER=threads>1?new MultiThreadsCounter():new SingleThreadCounter();
		
		for(int i=0;i<threads;i++) {
			new Thread() {
				@Override
				public void run() {
					try {
						int sessionId=connector.connect(remote,0);
						Session session=sessionSynchronizer.get(sessionId);
						for(int j=0;j<loop;j++) {
							EchoRequest req=new EchoRequest();
							req.seqid=COUNTER.getAndIncrement();
							req.data=DATA;
//							invocationSynchronizer.init(req.seqid);
							session.write(req);
//							EchoResponse res=invocationSynchronizer.get(req.seqid);
//							if(res==null) LOGGER.info("err|"+req.seqid); else LOGGER.info("res|"+req.seqid);
							if(req.seqid%batch==0) sleep(sleep);
						}
					} catch(Throwable t) {
						System.err.print("thread run() error");
						t.printStackTrace();
					}
				}
			}.start();
		}
	}

}
