package com.qq.taf.net.file.test;

import java.net.InetSocketAddress;
import java.nio.ByteBuffer;

import com.qq.jutil.j4log.Logger;
import com.qq.taf.net.MessageReceiver;
import com.qq.taf.net.Handler;
import com.qq.taf.net.Session;
import com.qq.taf.net.file.NetAcceptor;
import com.qq.taf.net.file.NetByteBuffer;
import com.qq.taf.net.file.NetFileBuffer;
import com.qq.taf.net.file.NetFilter;
import com.qq.taf.net.file.NetFilterEncodeProduct;
import com.qq.taf.net.file.NetMixBuffer;
import com.qq.taf.net.file.NetSession;

public class EchoServer {
	
	public static void main(String[] args) {
		String ip=System.getProperty("ip","127.0.0.1");
		int port=Integer.parseInt(System.getProperty("port","12345"));
		InetSocketAddress bind=new InetSocketAddress(ip,port);
		int handleThreads=0;
		
		int size=32;
		final byte[] DATA=new byte[size];
		for(int i=0;i<size;i++) DATA[i]='O';
		
		final Logger LOGGER=Logger.getLogger("EchoServer"); 
		
		new NetAcceptor(
			"EchoServer",
			new NetFilter() {
				public void encode(NetSession session, Object object,NetFilterEncodeProduct product) throws Throwable {
					EchoResponse res=(EchoResponse)object;
					NetFileBuffer buffer = new NetFileBuffer("e:/data", 1L, (int)(8-1) );
					ByteBuffer bb = ByteBuffer.allocate(4+4);
					bb.putInt(8-1+8).putInt(res.seqid).flip();
					NetByteBuffer nbb = new NetByteBuffer(bb);
					product.add((NetSession)session, new NetMixBuffer(nbb, buffer));
				}
				
				public void decode(Session session, ByteBuffer buffer,MessageReceiver product) throws Throwable {
					ByteBuffer now=buffer;
					ByteBuffer prev=(ByteBuffer)session.getAttribute("buff");
					if(prev!=null) {
						session.removeAttribute("buff");
						now=(ByteBuffer)ByteBuffer.allocate(prev.remaining()+buffer.remaining()).put(prev).put(buffer).flip();
					}
					while(now.remaining()>4 && now.remaining()>=now.getInt(now.position())) {
						int len=now.getInt();
						int seqid=now.getInt();
						byte[] data=new byte[len-4-4];
						now.get(data);
						EchoRequest req=new EchoRequest();
						req.seqid=seqid;
						req.data=data;
						product.add(session, req);
					} 
					if(now.hasRemaining()) {
						ByteBuffer succ=(ByteBuffer)ByteBuffer.allocate(now.remaining()).put(now).flip();
						session.setAttribute("buff", succ);
					}
				}
			},
			new Handler() {
				public void messageRecieved(Session session, Object message) throws Throwable {
					EchoRequest req=(EchoRequest)message;
					EchoResponse res=new EchoResponse();
					res.seqid=req.seqid;
					res.data=DATA;
					session.write(res);
					LOGGER.info(req.seqid+"");
				}
				public void exceptionCaught(Session session, Throwable exception) throws Throwable {
					session.close(true);
				}
			},
			0,
			handleThreads
		).bind(bind, 0).listen();
		
		System.out.println("EchoServer ...");
	}

}
