package com.qq.taf.net.udp;

import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 
 * @author easezhang
 *
 * @param <K>
 * @param <V>
 */
public class ExpireMap<K, V extends Expirable> {
	
	private final Map<K, V> map;
	
	private final static int DEFAULT_CLEAN_INTERVAL = 10;
	
	private final ExpireListener<V> listener;
	
	private final Expirer expirer;
	
	private final int cleanInterval;
	
	public ExpireMap(ExpireListener<V> listener) {
		this(listener, DEFAULT_CLEAN_INTERVAL);
	}
	
	public ExpireMap(ExpireListener<V> listener, int cleanInterval) {
		map = new ConcurrentHashMap<K, V>();
		this.listener = listener;
		this.cleanInterval = cleanInterval;
		expirer = new Expirer();
		expirer.start();
	}
	
	public V put(K key, V value) {
		return map.put(key, value);
	}
	
	public V remove(K key) {
		return map.remove(key);
	}
	
	public void clear() {
		map.clear();
	}
	
	public V get(K key) {
		return map.get(key);
	}
	private class Expirer extends Thread {
		public Expirer() {
			super("ExpireMap-Expirer");
		}
		public void run() {
			while(true) {
				try {
					Set<K> keys = map.keySet();
					V value = null;
					for(K key:keys) {
						value = map.get(key);
						if(value!=null && value.isExpired()) {
							map.remove(key);
							listener.onExpire(value);
						}
					}
					sleep(cleanInterval);
				}
				catch(Throwable t) {
					t.printStackTrace();
				}
			}
		}
	}
}
