package com.qq.taf.net.udp;

import java.net.SocketAddress;
import java.util.ArrayList;


/**
 * Session回收器，非线程安全类
 * @author easezhang
 *
 */
public class SessionRecycler {
	
	private ExpireMap<ArrayList<SocketAddress>, UDPSession> sessions;
	
	private final static int DEFAULT_SERVER_SESSION_CLEAN_INTERVAL = 10;
	
	private ExpireListener<UDPSession> expireListener;
	
	private static SessionRecycler DEFAULT_SERVER_SESSION_RECYCLER ;
	
	private SessionRecycler(ExpireListener<UDPSession> listner, int cleanTime) {
		if(cleanTime <= 0) {
			cleanTime = DEFAULT_SERVER_SESSION_CLEAN_INTERVAL;
		}
		expireListener = listner;
		sessions = new ExpireMap<ArrayList<SocketAddress>, UDPSession>(expireListener, cleanTime);
	}
	
	public static SessionRecycler getServerSessioinRecycler() {
		if(DEFAULT_SERVER_SESSION_RECYCLER == null) {
			synchronized(SessionRecycler.class) {
				if(DEFAULT_SERVER_SESSION_RECYCLER == null) {
					DEFAULT_SERVER_SESSION_RECYCLER = new SessionRecycler(new ServerSessionExpireListener(), DEFAULT_SERVER_SESSION_CLEAN_INTERVAL);
				}
			}
		}
		return DEFAULT_SERVER_SESSION_RECYCLER;
	}
	
	private ArrayList<SocketAddress> getKey(SocketAddress local, SocketAddress remote)  {
		ArrayList<SocketAddress> key = new ArrayList<SocketAddress>(2);
		key.add(local);
		key.add(remote);
		return key;
	}
	
	public UDPSession recycleSession(SocketAddress local, SocketAddress remote) {
		return sessions.get(getKey(local, remote));
	}
	
	public UDPSession addSession(SocketAddress local, SocketAddress remote, UDPSession ss) {
		return sessions.put(getKey(local, remote), ss);
	}
}
