package com.qq.taf.net.udp;

import java.net.SocketAddress;
import java.nio.channels.DatagramChannel;

import com.qq.taf.net.HandlerExecutor;

public interface UDPProcessor {
	
	/**
	 * UDP处理器的启动，源于它的懒初始化
	 * @param handlerExecutor
	 */
	public void start(HandlerExecutor handlerExecutor);
	
	/**
	 * 注册通道到selector
	 * @param dc
	 * @return
	 * @throws Throwable
	 */
	//public SelectionKey regist(DatagramChannel dc) throws Throwable;
	
	/**
	 * 关闭Processor，Processor资源回收
	 */
	public void close();
	
	/**
	 * 调度要写的Session
	 * @param session
	 * @return
	 */
	public boolean scheduleWrite(UDPSession session);
	
	/**
	 * 销毁Session持有的资源
	 * @param session
	 */
	public void scheduleCancel(UDPSession session);
	
	/**
	 * 调度连接远程channel
	 * @param dc 本地channel
	 * @param remote 远程channel
	 * @param sessionTimeOut session超时
	 * @param connectTimeOut 连接超时
	 * @return UDPSession
	 * @throws Throwable
	 */
	public int scheduleConnect(DatagramChannel dc, SocketAddress remote, int sessionTimeOut, int sessionId);

}
