package com.qq.taf.proxy;

import java.util.ArrayList;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.CopyOnWriteArrayList;

import com.qq.taf.StatFPrxHelper;
import com.qq.taf.StatMicMsgHead;
import com.qq.taf.StatSampleMsg;
import com.qq.taf.cnst.JCEMESSAGETYPESAMPLE;

/**
 * 
 * <b>功能：ProxyStatInfo的集合</b><br>
 * <br>
 * <b>完整路径：</b> com.qq.taf.proxy.ProxyStatManager <br>
 * <b>创建日期：</b> 2013-4-19 下午3:06:26 <br>
 * 
 * @author <a href="mailto:markzhang@tencent.com">markzhang</a><br>
 *         <a href="http://www.tencent.com">Shenzhen Tencent Co.,Ltd.</a>
 * @version 1.0, 2013-4-19
 * @since MQzone V1.0
 */
public class ProxyStatManager {
	
	
	private ProxyStatInfo statTool = new ProxyStatInfo();
	
	private final static int MAX_SAMPLE_MSGS = 1000;
	
	private ConcurrentHashMap<ServantProxy, ProxyStatInfo> proxyStats = new ConcurrentHashMap<ServantProxy, ProxyStatInfo>();
	
	private ConcurrentHashMap<ServantProxy, ConcurrentLinkedQueue<StatSampleMsg>> proxySamples = new ConcurrentHashMap<ServantProxy, ConcurrentLinkedQueue<StatSampleMsg>>();
	
	private CopyOnWriteArrayList<Integer> timeStatInterv = new CopyOnWriteArrayList<Integer>();
	
			
	private static ProxyStatManager instance = new ProxyStatManager();
	
	public static ProxyStatManager getInstance() {
		return instance;
	}
		
	private ProxyStatManager() {
		timeStatInterv.addAll(ProxyStatInfo.DefaultTimeStatInterv);
	}
	
	public void resetStatInterv() {
		timeStatInterv.clear();
		timeStatInterv.addAll(ProxyStatInfo.DefaultTimeStatInterv);
		setPointStatInterv();
	}
	
	public CopyOnWriteArrayList<Integer> getStatIntervals() {
		return timeStatInterv;
	}
	
	public ProxyStatInfo addStatTool(ServantProxy proxy) {
		ProxyStatInfo statTool = proxyStats.get(proxy);
		if(statTool == null) {
			statTool = new ProxyStatInfo();
			proxyStats.putIfAbsent(proxy, statTool);
		}
		return statTool;
	}
	
	public ProxyStatInfo getStatTool(ServantProxy proxy) {
		ProxyStatInfo tool = proxyStats.get(proxy);
		if(tool == null) {
			tool = addStatTool(proxy);
		}
		return tool;
	}
	
	public ProxyStatInfo getDefaultStatTool() {
		return statTool;
	}
	
	public ConcurrentHashMap<ServantProxy, ProxyStatInfo> getProxyStats() {
		return proxyStats;
	}
	
	public void setPointStatInterv() {
		for (Entry<ServantProxy, ProxyStatInfo> entry : proxyStats.entrySet()) {
			ServantProxy proxy = entry.getKey();
			proxy.taf_pointStatInterv(timeStatInterv);
		}
	}
	
	public void setPointStatInterv(ServantProxy proxy) {
		for (Entry<StatMicMsgHead, SendMsgStatBody> entry : getStatTool(proxy).getStats().entrySet()) {
			entry.getValue().setPointStatInterv(timeStatInterv);
		}
	}
	
	public void addStatInterv(int interv) {
		boolean add = false;
		for (int i = 0; i < timeStatInterv.size(); i++) {
			if (interv == timeStatInterv.get(i)) {
				return;
			}
			if (interv < timeStatInterv.get(i)) {
				timeStatInterv.add(i, interv);
				add = true;
				break;
			}
		}
		if (!add) {
			timeStatInterv.add(interv);
		}
		setPointStatInterv();
	}
	
	public void addSample(ServantProxy proxy, String remoteHost, String processName, SendMessage msg) {
		if(msg.isMsgType(JCEMESSAGETYPESAMPLE.value)) {
			StatSampleMsg sample=msg.sample;
			if(sample!=null) {
				sample.slaveIp=remoteHost;
				sample.masterName=processName;
				proxySamples.putIfAbsent(proxy, new ConcurrentLinkedQueue<StatSampleMsg>());
				if(proxySamples.get(proxy).size() > MAX_SAMPLE_MSGS) {
					TafLoggerCenter.error("can not add sample for" + proxy.taf_proxyName()+", queue size>"+MAX_SAMPLE_MSGS);
					return;
				}
				proxySamples.get(proxy).add(sample);
			}
		}
	}
	
	public void doSample(ServantProxy proxy) {
		ConcurrentLinkedQueue<StatSampleMsg> samples = proxySamples.get(proxy);
		if(samples == null) {
			return;
		}
		int sampleSize = samples.size();
		if ( sampleSize > 0 ) {
			int maxArrsySize = 10 ;
			int i = 0 ;
			int j = 0;
			StatFPrxHelper helper = proxy.getStatPrxPr();
			StatSampleMsg tmpMsg = null;
			ArrayList<StatSampleMsg> msgs = new ArrayList<StatSampleMsg>(maxArrsySize);
			while ( i < sampleSize ) {
				j = 0;
				while(j<maxArrsySize) {
					tmpMsg = samples.poll();
					if(tmpMsg!=null) {
						msgs.add(tmpMsg);
						++i;
						++j;
					}
					else {
						break;
					}
				}
				if(helper != null) {
					helper.reportSampleMsg(msgs);
				}
				msgs.clear();
			}
		}
	}
	
	public int getDefaultStatSize() {
		return this.statTool.size();
	}
	
}
