package com.qq.taf.proxy;

import java.util.ArrayList;

import com.qq.jutil.string.StringUtil;
import com.qq.taf.proxy.conn.EndPointInfo;

/**
 * proxy的配置文件 内部使用 调用者不用关心
 * 
 * @author albertzhu
 * 
 */
public class PrxConfig {
	private int hashCode;
	/**
	 * 是否是从配置中心获取的
	 */
	protected boolean fromRegisterCenter = false;

	String objectName;

	int refreshEndpointInterval;
	int asyncTimeoutMill;
	int syncTimeoutMill;
	String queryEndpoints;
	/**
	 * 在异步调用的时候防止冲击服务器端 有多少个未回应信息前不继续发送 设置为-1则不生效
	 */
	int availableKey = -1;
	TafThreadPool tafThreadPool;

	int serviceConnNum = 1;

	public ServiceInfos serviceInfos = null;

	public String locatorName = "";

	public String statName = "";

	int readBufferSize;
	int writeBufferSize;
	
	boolean enableSet;
	

	long idleStatusInteval =  60000*2; //2分钟没有操作连接就关闭

	String setDivision = "";
	String setName = "";	
	
	String setArea = "";
	
	String setId = "";
	
	
	public long getIdleStatusInteval() {
		return idleStatusInteval;
	}

	public void setIdleStatusInteval(long idleStatusInteval) {
		this.idleStatusInteval = idleStatusInteval;
	}
	
	public PrxConfig(String objectName, TafThreadPool tafThreadPool, int hashCode) {
		this.objectName = objectName;
		this.tafThreadPool = tafThreadPool;
		serviceInfos = new ServiceInfos(objectName);
		this.hashCode = hashCode;
	}

	public int getAsyncTimeoutMill() {
		return asyncTimeoutMill;
	}

	public void setAsyncTimeoutMill(int timeoutMill) {
		this.asyncTimeoutMill = timeoutMill;
	}

	public int getSyncTimeoutMill() {
		return syncTimeoutMill;
	}

	public void setSyncTimeoutMill(int syncTimeoutMill) {
		this.syncTimeoutMill = syncTimeoutMill;
	}

	public void setQueryEndpoints(String queryEndpoints, int serviceConnNum) {
		serviceInfos.setServices(queryEndpoints, syncTimeoutMill, asyncTimeoutMill, serviceConnNum);
		this.queryEndpoints = queryEndpoints;
	}

	public String toString() {
		return serviceInfos.getAllServicesFullString();
	}

	/**
	 * 得到缓存服务器信息需要的字符串
	 * @return
	 */
	public String getEndPoingListString() {
		return serviceInfos.getAllServicesStoreString();
	}

	
	public void setServiceInfos(ServiceInfos infos) {
		this.serviceInfos = infos;
	}	

	public static Pair<ArrayList<EndPointInfo>, ArrayList<EndPointInfo>> parseEndPointConfig(String objectName,
			int syncInvokeTimeout, int asyncInvokeTimeout, int serviceConnNum, String src) {
		ArrayList<EndPointInfo> activeEpList = new ArrayList<EndPointInfo>();
		ArrayList<EndPointInfo> inactiveEpList = new ArrayList<EndPointInfo>();
		String[] items = StringUtil.split(src, ":");
		for (String item : items) {
			EndPointInfo node = EndPointInfo.parseNode(objectName, item, syncInvokeTimeout,
					asyncInvokeTimeout, serviceConnNum);
			if (node.isActive()) {
				activeEpList.add(node);
			} else {
				inactiveEpList.add(node);
			}
		}
		Pair<ArrayList<EndPointInfo>, ArrayList<EndPointInfo>> p = new Pair<ArrayList<EndPointInfo>, ArrayList<EndPointInfo>>();
		p.k = activeEpList;
		p.v = inactiveEpList;
		return p;
	}

	@Override
	public boolean equals(Object obj) {
		return obj instanceof PrxConfig && ((PrxConfig)obj).hashCode == hashCode;
	}

	@Override
	public int hashCode() {
		return hashCode;
	}

	public String getObjectName() {
		return objectName;
	}

	public int getRefreshEndpointInterval() {
		return refreshEndpointInterval;
	}

	public void setRefreshEndpointInterval(int refreshEndpointInterval) {
		this.refreshEndpointInterval = refreshEndpointInterval;
	}

	public int getAvailableKey() {
		return availableKey;
	}

	public void setAvailableKey(int availableKey) {
		this.availableKey = availableKey;
	}

	public TafThreadPool getTafThreadPool() {
		return tafThreadPool;
	}

	public int getReadBufferSize() {
		return readBufferSize;
	}

	public void setReadBufferSize(int readBufferSize) {
		this.readBufferSize = readBufferSize;
	}

	public int getWriteBufferSize() {
		return writeBufferSize;
	}

	public void setWriteBufferSize(int writeBufferSize) {
		this.writeBufferSize = writeBufferSize;
	}

	public boolean isEnableSet() {
		return enableSet;
	}

	public void setEnableSet(boolean enableSet) {
		this.enableSet = enableSet;
	}

	public String getSetName() {
		return setName;
	}

	public void setSetName(String setName) {
		this.setName = setName;
	}
	
	public void setDivision(String division) {
		this.setDivision = division;
		if(division!=null) {
			String [] tmpStrs = division.split(".");
			if(tmpStrs.length >= 3) {
				setName = tmpStrs[0];
				setArea = tmpStrs[1];
				setId = tmpStrs[2];
			}
			else {
				setName = "";
				setArea = "";
				setId = "";
			}
		}
		
	}
	
	public String getSetDivision() {
		return setDivision;
	}

	public String getSetArea() {
		return setArea;
	}

	public String getSetId() {
		return setId;
	}
}
