package com.qq.taf.proxy;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.WriteLock;

import com.qq.jutil.string.StringUtil;
import com.qq.taf.EndpointF;
import com.qq.taf.proxy.conn.ServantEndPoint;
import com.qq.taf.proxy.conn.EndPointInfo;

/**
 * 存放registerCenter返回的service信息
 * 
 * @author albertzhu
 * 
 */
public class ServiceInfos implements IServicesInfo, Cloneable {

	//分组的所有EndPointInfo
	HashMap<Integer, ArrayList<EndPointInfo>> gridServices = new HashMap<Integer, ArrayList<EndPointInfo>>();

	//分组的活跃的EndPointInfo
	HashMap<Integer, ArrayList<EndPointInfo>> gridActiveServices = new HashMap<Integer, ArrayList<EndPointInfo>>();
	
	//活动的EndPointInfo
	ArrayList<EndPointInfo> activeServices = new ArrayList<EndPointInfo>();

	//不活动的EndPointInfo
	ArrayList<EndPointInfo> inactiveServices = new ArrayList<EndPointInfo>();

	//所有的EndPointInfo
	ArrayList<EndPointInfo> allServices = new ArrayList<EndPointInfo>();
	
	//活跃的key
	private HashSet<String> activeServiceKeys = new HashSet<String>();

	ReentrantReadWriteLock rwl = new ReentrantReadWriteLock();

	ReadLock rl = rwl.readLock();
	WriteLock wl = rwl.writeLock();

	String objectName;

	public ServiceInfos(String objectName) {
		this.objectName = objectName;
	}

	//EndPointInfo换组
	public void setServiceGrid(ServantEndPoint service, int newGridValue) {
		try {
			wl.lock();
			for (EndPointInfo e : allServices) {
				if (e.equals(service.getInfo())) {
					int oldGrid = e.getGrid();
					gridServices.get(oldGrid).remove(e);
					gridActiveServices.get(oldGrid).remove(e);
					e.setGrid(newGridValue);
					if ( null == gridServices.get(newGridValue) ) {
						gridServices.put(newGridValue, new ArrayList<EndPointInfo>());
					}
					if (!gridServices.get(newGridValue).contains(e)) {
						gridServices.get(newGridValue).add(e);
					}
					if ( null == gridActiveServices.get(newGridValue) ) {
						gridActiveServices.put(newGridValue, new ArrayList<EndPointInfo>());
					}
					if (!gridActiveServices.get(newGridValue).contains(e)) {
						gridActiveServices.get(newGridValue).add(e);
					}
					TafLoggerCenter.info(objectName + " setServiceGrid,old gridValue is "+oldGrid+", now gridValue is " + newGridValue
							+" "+ service.getInfo());
					//TODO 这里不需要break?
				}
			}
		} finally {
			TafLoggerCenter.info(objectName + " " + service.getInfo() + " setServiceGrid gridServices:" + gridServices
					+ " gridActiveServices:" + gridActiveServices);
			wl.unlock();
		}
	}

	public void setReconnInterv(int reconnInterv) {
		try {
			wl.lock();
			for (EndPointInfo e : allServices) {
				e.setFailedWaitTimeSecond(reconnInterv);
			}
		} finally {
			wl.unlock();
		}
	}


	/* (non-Javadoc)
	 * @see com.qq.taf.proxy.IServicesInfo#getActiveServices()
	 */
	public List<EndPointInfo> getActiveServices() {
		return activeServices;
	}
	
	public int getActiveServicesSize() {
		return activeServices.size();
	}
	
	
	public HashSet<String> getActiveServiceKeys() {
		return activeServiceKeys;
	}
	

	/* (non-Javadoc)
	 * @see com.qq.taf.proxy.IServicesInfo#getInactiveServices()
	 */
	public List<EndPointInfo> getInActiveServices() {
		return inactiveServices;
	}

	/* (non-Javadoc)
	 * @see com.qq.taf.proxy.IServicesInfo#getAllServices()
	 */
	public List<EndPointInfo> getAllServices() {
		return allServices;
	}
	
	
	public String getAllServicesFullString() {
		StringBuilder sb = new StringBuilder();
		sb.append(getObjectName()).append("@");
		ArrayList<EndPointInfo> tempList = allServices;
		for (int i = 0; i < tempList.size(); i++) {
			EndPointInfo node = tempList.get(i);
			if (i > 0) {
				sb.append(":");
			}
			sb.append(node.toFullString());
		}
		return sb.toString();
	}

	/**
	 * 得到缓存服务器信息需要的字符串
	 * @return
	 */
	public String getAllServicesStoreString() {
		StringBuilder sb = new StringBuilder();
		sb.append(getObjectName()).append("@");
		ArrayList<EndPointInfo> tempList = allServices;			
		for (int i = 0; i < tempList.size(); i++) {
			EndPointInfo node = tempList.get(i);
			if (i > 0) {
				sb.append(":");
			}
			sb.append(node.getStoreString());
		}
		return sb.toString();
	}	
	

	/**
	 * 服务器是否只有一种状态
	 * 
	 * @return
	 */
	public boolean isNoGrid() {
		try {
			rl.lock();
			return gridServices.size() == 1;
		} finally {
			rl.unlock();
		}
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(objectName);
		for (EndPointInfo e : allServices) {
			sb.append(";" + e.toFullString());
		}
		return sb.toString();
	}

	public void setServices(String queryEndpoints, int syncInvokeTimeout, int asyncInvokeTimeout, int serviceConnNum) {
		ArrayList<EndPointInfo> activeEpList = new ArrayList<EndPointInfo>();
		ArrayList<EndPointInfo> inactiveEpList = new ArrayList<EndPointInfo>();
		String[] items = StringUtil.split(queryEndpoints, ":");
		for (String item : items) {
			EndPointInfo node = EndPointInfo.parseNode(objectName,item, syncInvokeTimeout, asyncInvokeTimeout,
					serviceConnNum);
			if (node.isActive()) {
				activeEpList.add(node);
			} else {
				inactiveEpList.add(node);
			}
		}
		setServiceInfos(activeEpList, inactiveEpList);
	}

	private ArrayList<EndPointInfo> endpointF2EndPointInfo(ArrayList<EndpointF> oldFs, boolean active) {
		ArrayList<EndPointInfo> infos = new ArrayList<EndPointInfo>(oldFs.size());
		for (int i = 0; i < oldFs.size(); i++) {
			EndPointInfo info = new EndPointInfo(objectName,oldFs.get(i));
			info.setActive(active);
			infos.add(info);
		}
		Collections.sort(infos);
		return infos;
	}

	public void setServices(ArrayList<EndpointF> activeEp, ArrayList<EndpointF> inactiveEp) {
		setServiceInfos(endpointF2EndPointInfo(activeEp,true), endpointF2EndPointInfo(inactiveEp,false));
	}

	//设置服务信息
	void setServiceInfos(ArrayList<EndPointInfo> activeEp, ArrayList<EndPointInfo> inactiveEp) {

		HashMap<Integer, ArrayList<EndPointInfo>> _gridServices = new HashMap<Integer, ArrayList<EndPointInfo>>();
		HashMap<Integer, ArrayList<EndPointInfo>> _gridActiveServices = new HashMap<Integer, ArrayList<EndPointInfo>>();
		ArrayList<EndPointInfo> _activeServices = new ArrayList<EndPointInfo>();
		HashSet<String> _activeServiceKeys = new HashSet<String>();
		ArrayList<EndPointInfo> _inactiveServices = new ArrayList<EndPointInfo>();
		ArrayList<EndPointInfo> _allServices = new ArrayList<EndPointInfo>();

		for (EndPointInfo e : activeEp) {
			if (!_gridServices.containsKey(e.getGrid())) {
				_gridServices.put(e.getGrid(), new ArrayList<EndPointInfo>());
			}
			if (!_gridActiveServices.containsKey(e.getGrid())) {
				_gridActiveServices.put(e.getGrid(), new ArrayList<EndPointInfo>());
			}
			_gridServices.get(e.getGrid()).add(e);
			_gridActiveServices.get(e.getGrid()).add(e);
			_activeServices.add(e);
			_activeServiceKeys.add(e.getKey());
			_allServices.add(e);
		}
		for (EndPointInfo e : inactiveEp) {
			if (!_gridServices.containsKey(e.getGrid())) {
				_gridServices.put(e.getGrid(), new ArrayList<EndPointInfo>());
			}
			_gridServices.get(e.getGrid()).add(e);
			_inactiveServices.add(e);
			_allServices.add(e);
		}
		try {
			wl.lock();
			gridServices = _gridServices;
			gridActiveServices = _gridActiveServices;
			activeServices = _activeServices;
			activeServiceKeys = _activeServiceKeys;
			inactiveServices = _inactiveServices;
			allServices = _allServices;
		} finally {
			StringBuilder sb = new StringBuilder();
			sb.append(objectName + " setServices gridServices:" + gridServices + " gridActiveServices:"
					+ gridActiveServices);
			sb.append(" allServices: ");
			for ( EndPointInfo i : allServices) {
				sb.append(i+";");
			}
			TafLoggerCenter.debug(sb.toString());
			wl.unlock();
		}
	}
	
	public String getObjectName() {
		return objectName;
	}

	public void setObjectName(String objectName) {
		this.objectName = objectName;
	}

	@Override
	public EndPointInfo getGridActiveService(int grid, int seq) {
		try {
			rl.lock();
			List<EndPointInfo> list = gridActiveServices.get(grid);
			if(list == null||list.size() == 0) {return null;}
			else {
				return list.get(Math.abs(seq)%list.size());
			}
		} finally {
			rl.unlock();
		}
	}

	@Override
	public boolean hasGridServices(int grid) {
		try {
			rl.lock();
			List<EndPointInfo> list = gridServices.get(grid);
			return !(list == null||list.isEmpty());
		} finally {
			rl.unlock();
		}
	}
	
	public Object clone() {
		ServiceInfos newService = new ServiceInfos(this.objectName);
		try {
			rl.lock();
			newService.gridServices = deepClone(this.gridServices);
			newService.gridActiveServices = deepClone(this.gridActiveServices);
			newService.activeServices = (ArrayList<EndPointInfo>) this.activeServices.clone();
			newService.inactiveServices = (ArrayList<EndPointInfo>) this.inactiveServices.clone();
			newService.allServices = (ArrayList<EndPointInfo>) this.allServices.clone();
			newService.activeServiceKeys = (HashSet<String>) this.activeServiceKeys.clone();
		}
		finally {
			rl.unlock();
		}
		return newService;
	}
	
	//深度copy
	private HashMap<Integer, ArrayList<EndPointInfo>> deepClone(HashMap<Integer, ArrayList<EndPointInfo>> ref) {
		HashMap<Integer, ArrayList<EndPointInfo>> map = new HashMap<Integer, ArrayList<EndPointInfo>>(ref.size());
		Set<Map.Entry<Integer, ArrayList<EndPointInfo>>> set = ref.entrySet();
		for(Map.Entry<Integer, ArrayList<EndPointInfo>> entry : set) {
			int key = entry.getKey();
			ArrayList<EndPointInfo> value = (ArrayList<EndPointInfo>)entry.getValue().clone();
			map.put(key, value);
		}
		return map;
	}

}
