package com.qq.taf.proxy;

import java.util.ArrayList;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import com.qq.taf.EndpointF;
import com.qq.taf.QueryFPrxCallback;
import com.qq.taf.QueryFPrxHelper;
import com.qq.taf.cnst.JCESERVERSUCCESS;
import com.qq.taf.holder.JceArrayListHolder;
import com.qq.taf.proxy.conn.EndPointInfo;
import com.qq.taf.proxy.utils.TafUtils;

public class ServiceInfosRefresher implements Runnable {
	
	public static final ServiceInfosRefresher INSTANCE = new ServiceInfosRefresher();
	
	private ConcurrentHashMap<ServantProxy, ServiceInfos> configs;
	
	private ConcurrentHashMap<ServantProxy, Object> infos;
	
	private static int refreshInterval = 60*1000; 
	

	private ServiceInfosRefresher() {
		configs = new ConcurrentHashMap<ServantProxy, ServiceInfos>(128);
		infos = new ConcurrentHashMap<ServantProxy, Object>(128);
	}
	
	public static int getRefreshInterval() {
		return refreshInterval;
	}
	
	public static void setRefreshInterval(int intl) {
		refreshInterval = intl; 
	}
	
	public void addServantProxy(ServantProxy proxy) {
		if(!infos.contains(proxy)) {
			infos.putIfAbsent(proxy, new Object());
		}
		if(!configs.contains(proxy) && proxy.queryFPrx == null) {
			ServiceInfos serviceInfos = new ServiceInfos(proxy.prxConfig.objectName);
			
			//TODO 这里性能可以优化
			serviceInfos.setServiceInfos((ArrayList<EndPointInfo>)proxy.prxConfig.serviceInfos.getActiveServices(), (ArrayList<EndPointInfo>)proxy.prxConfig.serviceInfos.getInActiveServices());
			configs.putIfAbsent(proxy, serviceInfos);
		}
	}
	
	public void asyncGetServiceInfos(ServantProxy proxy) {
		if(proxy.canAsyncRefresh()) {
			asyncRefreshConfig(proxy, proxy.queryFPrx, proxy.prxConfig.objectName);
			proxy.setProxyAyncRefreshTime();
		}
		else {
			TafLoggerCenter.info(proxy.prxConfig.objectName+" can not async refresh config for refreshing in 2s or queryPrx is null");
		}
	}
	
	public ServiceInfos getServiceInfos(ServantProxy proxy) {
		ServiceInfos info = configs.get(proxy);
		if(info != null) {
			//防止引用内容被修改，需要额外拷贝一份
			return (ServiceInfos)info.clone();
		}
		else {
			synchronized(infos.get(proxy)) {
				info = configs.get(proxy);
				if(info == null) {
					if(!(proxy instanceof QueryFPrxHelper)) { //死循环问题
						info = refreshConfig(proxy, proxy.queryFPrx, proxy.prxConfig.objectName);
					}
					if(info != null) {
						configs.putIfAbsent(proxy, info);
					}
					else {
						TafLoggerCenter.error(proxy.serviceInfos.objectName+" sync get config is null");
					}
				}
			}
			if(info!=null) {
				return (ServiceInfos)info.clone();
			}
			return new ServiceInfos(proxy.serviceInfos.objectName);
		}
	}
	
	private void asyncRefreshConfig(ServantProxy proxy, QueryFPrxHelper queryFPrx, String objectName) {
		try {
			TafLoggerCenter.debug(objectName+" asyncRefreshConfig, "+proxy.prxConfig.enableSet+", "+proxy.prxConfig.setDivision+" "+proxy.prxConfig.hashCode());
			QueryServiceInfoCallback queryServiceCallBack = new QueryServiceInfoCallback(proxy, objectName);
			ayncFindObjectByQuery(proxy.prxConfig.enableSet, proxy.prxConfig.setDivision, queryFPrx, queryServiceCallBack, objectName);
			//queryFPrx.async_findObjectById4All(queryServiceCallBack, objectName);
		} catch (Exception e) {
			TafLoggerCenter.error(objectName + " asyncRefreshConfig error", e);
		}
	}
	
	@SuppressWarnings("unchecked")
	private ServiceInfos refreshConfig(ServantProxy proxy, QueryFPrxHelper queryFPrx, String objectName) {
		boolean querySucc = false;
		ServiceInfos info = null;
		try {
			TafLoggerCenter.debug(objectName+" refreshConfig, "+proxy.prxConfig.enableSet+", "+proxy.prxConfig.setDivision);
			JceArrayListHolder activeEp = new JceArrayListHolder();
			JceArrayListHolder inactiveEp = new JceArrayListHolder();
			int _ret = findObjectByQuery(proxy.prxConfig.enableSet, proxy.prxConfig.setDivision, queryFPrx, objectName, activeEp, inactiveEp); 
			if (_ret == JCESERVERSUCCESS.value) {
				info = parseRespEndPointInfo(objectName, (activeEp.getValue()), (inactiveEp.getValue()));
				querySucc = true;
			} else {
				TafLoggerCenter.error(objectName + " findObjectByQuery error , return " + _ret);
			}
		} catch (Exception e) {
			TafLoggerCenter.error(objectName + " findObjectByQuery error " + e, e);
		}
		if (!querySucc) {
			PrxConfig servantConfig = proxy.prxConfig;
			Pair<ArrayList<EndPointInfo>, ArrayList<EndPointInfo>> p = TafUtils.getServerInfoData(servantConfig
					.getObjectName(), servantConfig.syncTimeoutMill, servantConfig.asyncTimeoutMill,
					servantConfig.serviceConnNum);
			//可用列表为空 认为全部可用
			if ( null != p  ) {
				info = parseRespServicePointInfo(objectName, p.k, p.v);
			}
		}
		return info;

	}
	
	private int findObjectByQuery(boolean isOpenSet, String setId, QueryFPrxHelper queryFPrx, String objectName, JceArrayListHolder activeEp, JceArrayListHolder inactiveEp) {
		if(isOpenSet) {
			return queryFPrx.findObjectByIdInSameSet(objectName, setId, activeEp, inactiveEp);
		}
		else {
			return queryFPrx.findObjectByIdInSameGroup(objectName, activeEp, inactiveEp);
		}
	}
	
	private void ayncFindObjectByQuery(boolean isOpenSet, String setId, QueryFPrxHelper queryFPrx, QueryServiceInfoCallback queryServiceCallBack, String objectName) {
		if(isOpenSet) {
			queryFPrx.async_findObjectByIdInSameSet(queryServiceCallBack, objectName, setId);
		}
		else {
			queryFPrx.async_findObjectByIdInSameGroup(queryServiceCallBack, objectName);
		}
	}
	/**
	 * 解析注册中心返回的endPoint列表并刷新proxyConns
	 * 
	 * @param activeEp
	 * @param inactiveEp
	 */
	private ServiceInfos parseRespEndPointInfo(String objectName, ArrayList<EndpointF> activeEp, ArrayList<EndpointF> inactiveEp) {
		//可用列表为空 认为全部可用
		if(activeEp.size()>0) {
			//完全相信从主控拉取的列表 
			ServiceInfos info = new ServiceInfos(objectName);
			info.setServices(activeEp, inactiveEp);
			TafLoggerCenter.debug(objectName+" cacheServerData get config from " + info);
			return info;
		}
		else {
			//如果本次取得的活动列表为0，暂用上次取得的活动列表
			TafLoggerCenter.error(objectName+" callback_findObjectById4All get config from register active size = 0");
			return null;
		}
	}
	
	private ServiceInfos parseRespServicePointInfo(String objectName, ArrayList<EndPointInfo> activeEp, ArrayList<EndPointInfo> inactiveEp) {
		//完全相信主控
		if(activeEp.size()>0) {
			ServiceInfos info = new ServiceInfos(objectName);
			info.setServiceInfos(activeEp, inactiveEp);
			TafLoggerCenter.debug(objectName+" cacheServerData get config from " + info);
			return info;
		}
		else {
			//如果本次取得的活动列表为0，暂用上次取得的活动列表
			TafLoggerCenter.error(objectName+" callback_findObjectById4All get config from register active size = 0");
			return null;
		}
	}
	
	
	/**
	 * 从register中心查询服务器信息的回调方法
	 * 
	 * @author albertzhu
	 * 
	 */
	class QueryServiceInfoCallback extends QueryFPrxCallback {
		
		private ServantProxy proxy;
		private String objectName;
		
		public QueryServiceInfoCallback(ServantProxy proxy, String objectName) {
			this.proxy = proxy;
			this.objectName = objectName;
		}
		
		@Override
		public void callback_findObjectById_exception(int ret) {
			TafLoggerCenter.info(objectName+ " callback_findObjectById_exception " + ret);
		}


		@Override
		public void callback_findObjectById4All_exception(int ret) {
			TafLoggerCenter.info(objectName + " callback_findObjectById4All_exception " + ret);
		}

		@Override
		public void callback_findObjectById(ArrayList<EndpointF> _ret) {
			TafLoggerCenter.info(objectName + "callback_findObjectById is not used");
			
		}

		@Override
		public void callback_findObjectById4All(int _ret,
				ArrayList<EndpointF> activeEp, ArrayList<EndpointF> inactiveEp) {
			if (_ret == JCESERVERSUCCESS.value) {
				refreshRouter(activeEp, inactiveEp);
				// 存储从register取回的结果
				TafUtils.setServerInfoData(objectName, proxy.prxConfig.getEndPoingListString());
				TafLoggerCenter.debug(objectName+" callback_findObjectById4All finish");
			} else {
				refreshByLocal();
				TafLoggerCenter.error(objectName + "async findObjectById4All error , return " + _ret);
			}
			
		}

		@Override
		public void callback_findObjectByIdInSameGroup(int _ret,
				ArrayList<EndpointF> activeEp, ArrayList<EndpointF> inactiveEp) {
			if (_ret == JCESERVERSUCCESS.value) {
				refreshRouter(activeEp, inactiveEp);
				// 存储从register取回的结果
				TafUtils.setServerInfoData(objectName, proxy.prxConfig.getEndPoingListString());
				TafLoggerCenter.debug(objectName+" callback_findObjectByIdInSameGroup finish");

			} else {
				refreshByLocal();
				TafLoggerCenter.error(objectName + "async findObjectByIdInSameGroup error , return " + _ret);
			}
			
		}

		@Override
		public void callback_findObjectByIdInSameGroup_exception(int _iRet) {
			TafLoggerCenter.info(objectName + " callback_findObjectByIdInSameGroup_exception " + _iRet);
			
		}

		private void refreshRouter(ArrayList<EndpointF> activeEp, ArrayList<EndpointF> inactiveEp) {
			ServiceInfos info = parseRespEndPointInfo(objectName, activeEp, inactiveEp);
			if(info != null) {
				configs.put(proxy, info);
				ServiceInfos copyedInfo = (ServiceInfos)info.clone();
				proxy.connectionManager.refreshEndPoints(copyedInfo);
				proxy.prxConfig.setServiceInfos(copyedInfo);
				TafLoggerCenter.debug(objectName+" async refresh serviceinfos:"+info);
			}
		}
		
		private void refreshRouter2(ArrayList<EndPointInfo> activeEp, ArrayList<EndPointInfo> inactiveEp) {
			ServiceInfos info = parseRespServicePointInfo(objectName, activeEp, inactiveEp);
			if(info != null) {
				configs.put(proxy, info);
				ServiceInfos copyedInfo = (ServiceInfos)info.clone();
				proxy.connectionManager.refreshEndPoints(copyedInfo);
				proxy.prxConfig.setServiceInfos(copyedInfo);
				TafLoggerCenter.debug(objectName+" async refresh serviceinfos:"+info);
			}
		}
		
		private void refreshByLocal() {
			PrxConfig proxyConfig = proxy.prxConfig;
			Pair<ArrayList<EndPointInfo>, ArrayList<EndPointInfo>> p = TafUtils.getServerInfoData(proxyConfig
					.getObjectName(), proxyConfig.syncTimeoutMill, proxyConfig.asyncTimeoutMill,
					proxyConfig.serviceConnNum);
			if(p != null) {
				refreshRouter2(p.k, p.v);
			}
		}
		
		@Override
		public void callback_findObjectByIdInSameStation(int _ret,
				ArrayList<EndpointF> activeEp, ArrayList<EndpointF> inactiveEp) {
			if (_ret == JCESERVERSUCCESS.value) {
				refreshRouter(activeEp, inactiveEp);
				// 存储从register取回的结果
				TafUtils.setServerInfoData(objectName, proxy.prxConfig.getEndPoingListString());
				TafLoggerCenter.debug(objectName+" callback_findObjectByIdInSameStation finish");

			} else {
				refreshByLocal();
				TafLoggerCenter.error(objectName + "async findObjectByIdInSameStation error , return " + _ret);
			}
		}

		@Override
		public void callback_findObjectByIdInSameStation_exception(int _iRet) {
			TafLoggerCenter.info(objectName + " callback_findObjectByIdInSameStation_exception " + _iRet);
			
		}

		@Override
		public void callback_findObjectByIdInSameSet(int _ret,
				ArrayList<EndpointF> activeEp, ArrayList<EndpointF> inactiveEp) {
			if (_ret == JCESERVERSUCCESS.value) {
				refreshRouter(activeEp, inactiveEp);
				// 存储从register取回的结果
				TafUtils.setServerInfoData(objectName, proxy.prxConfig.getEndPoingListString());
				TafLoggerCenter.debug(objectName+" callback_findObjectByIdInSameSet finish");

			} else {
				refreshByLocal();
				TafLoggerCenter.error(objectName + " async findObjectByIdInSameSet error , return " + _ret);
			}
		}

		@Override
		public void callback_findObjectByIdInSameSet_exception(int _iRet) {
			TafLoggerCenter.info(objectName + " callback_findObjectByIdInSameSet_exception " + _iRet);
			
		}

		@Override
		public void callback_findObjectById4Any(int _ret, ArrayList<EndpointF> activeEp,
				ArrayList<EndpointF> inactiveEp)
		{
			// TODO Auto-generated method stub
			
		}

		@Override
		public void callback_findObjectById4Any_exception(int _iRet)
		{
			// TODO Auto-generated method stub
			
		}
	}


	@Override
	public void run() {
		long begin = System.currentTimeMillis();
		final Set<ServantProxy> proxys = infos.keySet();
		for(ServantProxy proxy:proxys) {
			if(proxy.queryFPrx != null) {
				//没有指定ip端口的才刷新
				asyncRefreshConfig(proxy, proxy.queryFPrx, proxy.prxConfig.objectName);
			}
			else {
				TafLoggerCenter.info(proxy.prxConfig.objectName +" proxy.queryFPrx is null, can not refresh serviceInfo");
			}
		}
		TafLoggerCenter.info("ServiceInfosRefresher run(), use: " + (System.currentTimeMillis() - begin));					
	}
}
