package com.qq.taf.proxy.conn;

import java.net.InetSocketAddress;
import java.util.concurrent.LinkedBlockingQueue;

import com.qq.taf.net.Connector;
import com.qq.taf.net.Handler;
import com.qq.taf.net.Session;
import com.qq.taf.net.SessionIdGenerator;
import com.qq.taf.net.SessionSynchronizer;
import com.qq.taf.net.udp.UDPConnector;
import com.qq.taf.proxy.TafLoggerCenter;
import com.qq.taf.proxy.codec.JceCodec;

/**
 * 网络连接接口 
 * 根据请求打开连接并转发IoSession事件
 * 
 * @author albertzhu
 * 
 */
public class EndPointAdapter extends Handler {
	//没有限制最大数
	private LinkedBlockingQueue<ServantEndPoint> connectQueue = new LinkedBlockingQueue<ServantEndPoint>();
	private boolean running = true;
	
	private Connector socketConnector ;
	private UDPConnector datagramConnector ;
	private SessionSynchronizer sessionSynchronizer;

	public EndPointAdapter() {
		JceCodec jceCodec=new JceCodec(true);
		
		// !!! 消息处理器为EndPointAdapter本身
		socketConnector = new Connector("TafTcpClient", jceCodec, this, 0, 0);		
		datagramConnector = new UDPConnector("TafUdpClient", jceCodec, this, 0, 0);
		
		SessionIdGenerator sessionIdGenerator=new SessionIdGenerator();
		socketConnector.setSessionIdGenerator(sessionIdGenerator);
		datagramConnector.setSessionIdGenerator(sessionIdGenerator);
		sessionSynchronizer=new SessionSynchronizer(3000,512);
		
		Thread consumer = new Thread(new ConnectingWorker(),"ConsumerRequestWorker");
		consumer.start();
	}


	/**
	 * 开启连接 非阻塞方法 立即返回
	 * 
	 * @param connReq
	 */
	public void openConn(ServantEndPoint servantEndPoint) {
		connectQueue.offer(servantEndPoint);
	}
	
	public final static String listen_session_key = "taf_listen";
	
	//建立tcp、udp链接的线程
	class ConnectingWorker implements Runnable {
		public void run() {
			while (running) {
				ServantEndPoint endPoint = null;
				try {
					endPoint = connectQueue.take();
					EndPointInfo endPointInfo = endPoint.getInfo();
					int sessionId;
					if (endPointInfo.protocol.equals("tcp")) {
						sessionId = socketConnector.connect(new InetSocketAddress(endPointInfo.remoteHost, endPointInfo.remotePort), -1);//request.getSyncInvokeTimeout()
					} else if (endPointInfo.protocol.equals("udp")){
						sessionId = datagramConnector.connect(new InetSocketAddress(endPointInfo.remoteHost, endPointInfo.remotePort),-1);
					} else {
						throw new RuntimeException("unknown protocol "+endPointInfo.protocol);
					}
					Session session=sessionSynchronizer.get(sessionId);
					if(session!=null) {
						//设置session关联的ServantEndPoint
						session.setAttribute(listen_session_key, endPoint);
						
						try {
							endPoint.onSessionOpened(session);
						} catch(Throwable t) {
							TafLoggerCenter.error("onSessionOpen error",t);
							endPoint.onSessionOpenedFailed();
						}
					} else {
						TafLoggerCenter.error("error , can not open conn for "+endPointInfo+" ");
						endPoint.onSessionOpenedFailed();
					}
				}
				catch(Throwable th) {
					if(endPoint !=null) {
						endPoint.onSessionOpenedFailed();
						TafLoggerCenter.error("can not open conn for "+endPoint+" ", th);
					}
					else {
						TafLoggerCenter.error("can not open conn for error ", th);
					}
				}
			}
		}
	}


	// 以下为继承Handler的代码
	// //////////////////////////////////////////////
	@Override
	public void exceptionCaught(Session session, Throwable cause) throws Throwable {
		ServantEndPoint endPoint = getServantEndPoint(session);
		if(endPoint != null) {
			endPoint.onExceptionCaught(session, cause);
		}
		session.close(true);
	}

	@Override
	public void messageRecieved(Session session, Object message) throws Throwable {
		getServantEndPoint(session).onMessageReceived(session, message);
	}

	@Override
	public void sessionClosed(Session session) throws Throwable {
		ServantEndPoint endPoint = getServantEndPoint(session);
		if(endPoint != null) {		
			endPoint.onSessionClosed(session);
		}
		else {
			//UDPSession.close(true)时，会调用destroy()将Attribute清理掉
		}
	}

	@Override
	public void sessionOpened(Session session) throws Exception {
		sessionSynchronizer.put(session);
	}
	
	//取出Session对应的ServantEndPoint
	private ServantEndPoint getServantEndPoint(Session session) {
		return (ServantEndPoint) session.getAttribute(listen_session_key);
	}
}
