package com.qq.taf.proxy.conn;

import java.util.Date;
import java.util.concurrent.atomic.AtomicBoolean;

import com.qq.taf.QueryFPrxHelper;
import com.qq.taf.proxy.SendMsgStatBody;
import com.qq.taf.proxy.TafLoggerCenter;
import com.qq.taf.proxy.utils.Millis100TimeProvider;
import com.qq.taf.proxy.utils.TafUtils;

/**
 * 
 * <b>功能：proxy切换信息，因为EndPointInfo有很多更新操作，不能直接放到EndPointInfo里边</b><br>
 * <br>
 * <b>完整路径：</b> com.qq.taf.proxy.ProxySwitchInfo <br>
 * <b>创建日期：</b> 2013-4-19 下午6:02:24 <br>
 * 
 * @author <a href="mailto:markzhang@tencent.com">markzhang</a><br>
 *         <a href="http://www.tencent.com">Shenzhen Tencent Co.,Ltd.</a>
 * @version 1.0, 2013-4-19
 * @since MQzone V1.0
 */
public class ProxySwitchInfo
{
	private AtomicBoolean lastCallSucc = new AtomicBoolean();
	
	//超时统计开始的时间点
	private long timeout_startTime = Millis100TimeProvider.INSTANCE.currentTimeMillis() ;
	//连续失败次数
	private long frequenceFailInvoke = 0;
	private long frequenceFailInvoke_startTime = 0;
	
	private long timeoutCount = 0;
	private long failedCount = 0;
	private long succCount = 0;
	//网络链接超时
	private boolean netConnectTimeout = false;
	private boolean active = true;
	
	private long lastRetryTime = 0;

	
	public boolean getNetConnectTimeout()
	{
		return netConnectTimeout;
	}


	public AtomicBoolean getLastCallSucc()
	{
		return lastCallSucc;
	}


	public long getFrequenceFailInvoke()
	{
		return frequenceFailInvoke;
	}


	public long getTimeoutCount()
	{
		return timeoutCount;
	}


	public long getFailedCount()
	{
		return failedCount;
	}


	public long getSuccCount()
	{
		return succCount;
	}


	public void onCallFinished(ServantConnManager connManager, EndPointInfo endpointInfo, int callStatus) {
			
		//统计
		if ( callStatus == SendMsgStatBody.CallStatusSucc ) {
			frequenceFailInvoke = 0;
			frequenceFailInvoke_startTime = 0;
			lastCallSucc.set(true);
			netConnectTimeout = false;			
			succCount++;;
		} else if ( callStatus == SendMsgStatBody.CallStatusTimeout ) {
			if ( !lastCallSucc.get() ) {
				frequenceFailInvoke++;
			} else {
				lastCallSucc.set(false);
				frequenceFailInvoke = 1;
				frequenceFailInvoke_startTime = Millis100TimeProvider.INSTANCE.currentTimeMillis();
			}
			netConnectTimeout = false;			
			timeoutCount ++;
		} else if ( callStatus == SendMsgStatBody.CallStatusExec ) {
			if ( !lastCallSucc.get() ) {
				frequenceFailInvoke ++;
			} else {
				lastCallSucc.set(false);
				frequenceFailInvoke = 1;
				frequenceFailInvoke_startTime = Millis100TimeProvider.INSTANCE.currentTimeMillis();				
			}
			netConnectTimeout = false;			
			failedCount ++;
		} else if(callStatus == SendMsgStatBody.CallStatusNetConnectTimeout ) {
			netConnectTimeout = true;
		}
		
		if((timeout_startTime + connManager.getServantProxy().taf_timeout_interval()) < Millis100TimeProvider.INSTANCE.currentTimeMillis()) {
			timeoutCount = 0;
			failedCount = 0;
			succCount = 0;
			timeout_startTime = Millis100TimeProvider.INSTANCE.currentTimeMillis();
		}		
		
		if(active) {			
			//计算
			long totalCount = timeoutCount + failedCount + succCount;			
			if(timeoutCount >= connManager.getServantProxy().taf_min_timeout_invoke()) {
				//TODO 用乘法替换现在的除法
				double radio = TafUtils.div(timeoutCount, totalCount, 2) ;
				if ( radio > connManager.getServantProxy().taf_timeout_radio()) {
					active = false;
				}			
			}
			
			if(active) {
				//5秒内连续失败5次
				if(frequenceFailInvoke >= connManager.getServantProxy().taf_frequence_fail_invoke() 
						&& (frequenceFailInvoke_startTime + 5000) > Millis100TimeProvider.INSTANCE.currentTimeMillis()) {
					active = false;
				}
			}
			
			if(active) {
				if(netConnectTimeout) {
					active = false;
				}
			}			
		}
		else {
			if (callStatus == SendMsgStatBody.CallStatusSucc) {
				active = true;
			}			
		}
		
		if(TafUtils.developMode && connManager.proxy instanceof QueryFPrxHelper) {
			TafLoggerCenter.error("onCallFinished(" + callStatus + ") " + this + " " + endpointInfo);
		}
		
	}


	public boolean isActive()
	{
		return active;
	}


	public void setActive(boolean active)
	{
		this.active = active;
	}


	public long getLastRetryTime()
	{
		return lastRetryTime;
	}


	public void setLastRetryTime(long lastRetryTime)
	{
		this.lastRetryTime = lastRetryTime;
	}	
	
	
	public String toString() {
		StringBuilder build = new StringBuilder();
		build.append("lastCallSucc:").append(lastCallSucc.get()).append("|");
		build.append("timeout_startTime:").append(new Date(timeout_startTime)).append("|");
		build.append("frequenceFailInvoke:").append(frequenceFailInvoke).append("|");
//		build.append("frequenceFailInvoke_startTime:").append(new Date(frequenceFailInvoke_startTime)).append("|");
		
		build.append("timeoutCount:").append(timeoutCount).append("|");
		build.append("failedCount:").append(failedCount).append("|");
		build.append("succCount:").append(succCount).append("|");
		
		build.append("netConnectTimeout:").append(netConnectTimeout).append("|");
		build.append("active:").append(active).append("|");
//		build.append("lastRetryTime:").append(new Date(lastRetryTime));
				
		return build.toString();
	}
	
}
