package com.qq.taf.proxy.conn;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import com.qq.taf.QueryFPrxHelper;
import com.qq.taf.proxy.ServantProxy;
import com.qq.taf.proxy.TafLoggerCenter;
import com.qq.taf.proxy.conn.EndPointInfo;
import com.qq.taf.proxy.conn.ServantConnManager;
import com.qq.taf.proxy.utils.Millis100TimeProvider;
import com.qq.taf.proxy.utils.TafUtils;


public class SwitchInfoFactory
{
	//是否能够打印连接日志，每10条打一条ERROR，其他打印DEBUG
	private static long logTypeSeed = 0;
	private static Map<EndPointInfo, ProxySwitchInfo> switchMap = new ConcurrentHashMap<EndPointInfo, ProxySwitchInfo>();
	public static ProxySwitchInfo getSwitchInfo(EndPointInfo endpointInfo) {		
		ProxySwitchInfo sw = switchMap.get(endpointInfo);
		if(sw == null) {
			sw = new ProxySwitchInfo();
			switchMap.put(endpointInfo, sw);
		}
		return sw;
	}
	
	/**
	 * 
	 * <b>功能：是否可以链接</b><br>
	 * <br>
	 * <b>实现步骤：</b><br>
	 * <b>1.</b>  <br>
	 * <b>2.</b>  <br>
	 * @修改者 ~ , markzhang 2013-4-19
	 * @param update， 如果检查不通过，是否关闭链接和清零,返回是否需要关闭
	 */
	public static boolean canConnect(EndPointInfo info, ServantConnManager connManager) {
		ProxySwitchInfo sw = getSwitchInfo(info);
		boolean ret = true;
		if(!sw.isActive()) {
			if((sw.getLastRetryTime() + connManager.getServantProxy().taf_try_time_interval() * 1000) < Millis100TimeProvider.INSTANCE.currentTimeMillis()) {
				sw.setLastRetryTime(Millis100TimeProvider.INSTANCE.currentTimeMillis());
				ret = true;
			} else {
				ret = false;
			}
			
			//ServantProxy proxy = connManager.getServantProxy();			
			if(!ret) {
				//调试日志
				if(logTypeSeed % 10 == 5) {
					TafLoggerCenter.error("canConnect():ret:" + ret + " " + info + " " + sw + " " + Thread.currentThread().getName());
				}
				else {
					TafLoggerCenter.debug("canConnect():ret:" + ret + " " + info + " " + sw + " " + Thread.currentThread().getName());					
				}
				logTypeSeed++;
			}
			
		}
		return ret;
	}

	/**
	 * 
	 * <b>功能：</b><br>
	 * <br>
	 * <b>实现步骤：</b><br>
	 * <b>1.</b>  <br>
	 * <b>2.</b>  <br>
	 * @修改者 ~ , markzhang 2013-4-22
	 * @param endpointInfo
	 * @param callStatus void
	 */
	public static void onCallFinished(ServantConnManager connManager, EndPointInfo endpointInfo, int callStatus) {
		SwitchInfoFactory.getSwitchInfo(endpointInfo).onCallFinished(connManager, endpointInfo, callStatus);		
	}
}
